package com.tzld.piaoquan.recommend.feature.produce;

import com.tzld.piaoquan.recommend.feature.produce.model.DTSConfig;
import com.tzld.piaoquan.recommend.feature.produce.service.CMDService;
import com.tzld.piaoquan.recommend.feature.produce.service.DTSConfigService;
import com.tzld.piaoquan.recommend.feature.produce.service.ODPSService;
import com.tzld.piaoquan.recommend.feature.produce.service.RedisService;
import com.tzld.piaoquan.recommend.feature.produce.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;

import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Slf4j
public class ODPSToRedis {
    private static CMDService cmdService = new CMDService();
    private static ODPSService odpsService = new ODPSService();

    public static void main(String[] args) {

        log.info("args {}", JSONUtils.toJson(args));

        Map<String, String> argMap = cmdService.parse(args);
        if (MapUtils.isEmpty(argMap)) {
            log.error("args is empty");
            return;
        }
        String env = argMap.get("env");
        // ODPS
        DTSConfigService dtsConfigService = new DTSConfigService(env);
        DTSConfig config = dtsConfigService.getDTSConfig(argMap);
        if (config == null || !config.selfCheck()) {
            log.error("dts config error");
            return;
        }


        log.info("read odps");
        List<Map<String, String>> fieldValues = odpsService.read(config, argMap);
        log.info("odps count {}", fieldValues.size());
        if (CollectionUtils.isEmpty(fieldValues)) {
            return;
        }

        // RDD
        RedisService redisService = new RedisService(env);
        SparkConf sparkConf = new SparkConf()
                .setAppName("odps sync to redis");
        // .setMaster("local");
        JavaSparkContext jsc = new JavaSparkContext(sparkConf);
        JavaRDD<Map<String, String>> readData = jsc.parallelize(fieldValues);
        JavaRDD<Pair<String, String>> json = readData.map(
                f -> Pair.of(redisService.redisKey(f, config), JSONUtils.toJson(f))
        );

        int partitionNum = fieldValues.size() / 1000 + 1;
        log.info("sync redis");
        json.repartition(partitionNum).foreachPartition(iterator -> {
            redisService.mSet(iterator, config);
        });
    }

}
